@@ -100,7 +100,7 @@ class Block(object):
         else:
             attributes = ""
 
-        return "Block(%d %r%s)" % (self.slot, self.name, attributes)
+        return "Block(%r %r%s)" % (self.key, self.name, attributes)
 
     __repr__ = __str__
 
@@ -148,9 +148,13 @@ class Item(object):
         self.slot = slot
         self.name = name
 
-        # XXX
         self.key = (self.slot, secondary)
 
+    def __str__(self):
+        return "Item(%r %r)" % (self.key, self.name)
+
+    __repr__ = __str__
+
 block_names = [
     "air", # 0x0
     "stone",
@@ -535,7 +539,7 @@ _add_block(Block(76, "redstone-torch", orientation=(None, 5, 4, 3, 2, 1), dim=0)
 # Stone buttons are orientable.
 _add_block(Block(77, "stone-button", orientation=(None, None, 1, 2, 3, 4), dim=0))
 # Ice drops nothing, is replaced by springs, and dims by 3.
-_add_block(Block(79, "ice", drop=0, replace=8, dim=3))
+_add_block(Block(79, "ice", drop=0, replace=9, dim=3))
 # Clay drops 4 clay balls.
 _add_block(Block(82, "clay", drop=337, quantity=4))
 
@@ -648,3 +652,24 @@ armor_boots = (301, 305, 309, 313, 317)
 """
 List of slots of boots.
 """
+
+def parse_block(block):
+    """
+    Get the key for a given block/item.
+    """
+
+    try:
+        if block.startswith("0x") and (
+            (int(block, 16) in blocks) or (int(block, 16) in items)):
+            return (int(block, 16), 0)
+        elif (int(block) in blocks) or (int(block) in items):
+            return (int(block), 0)
+        else:
+            raise Exception("Couldn't find block id %s!" % block)
+    except ValueError:
+        if block in blocks:
+            return blocks[block].key
+        elif block in items:
+            return items[block].key
+        else:
+            raise Exception("Couldn't parse block %s!" % block)
