@@ -1,3 +1,4 @@
+from collections import defaultdict
 from itertools import chain
 from time import time
 
@@ -9,8 +10,9 @@ from zope.interface import implements
 
 from bravo.config import configuration
 from bravo.entity import entities
-from bravo.ibravo import (IAutomaton, IAuthenticator, ISeason,
-    ITerrainGenerator)
+from bravo.ibravo import (ISortedPlugin, IAutomaton, IAuthenticator, ISeason,
+    ITerrainGenerator, IUseHook, ISignHook, IDigHook, IPreBuildHook,
+    IPostBuildHook)
 from bravo.location import Location
 from bravo.packets.beta import make_packet
 from bravo.plugin import retrieve_named_plugins, retrieve_sorted_plugins
@@ -57,23 +59,13 @@ class BravoFactory(Factory):
         self.interface = configuration.getdefault(self.config_name, "host",
             "")
 
-    def startFactory(self):
-        log.msg("Initializing factory for world '%s'..." % self.name)
-
         self.world = World(self.name)
         self.world.factory = self
-        if configuration.has_option(self.config_name, "perm_cache"):
-            cache_level = configuration.getint(self.config_name, "perm_cache")
-            self.world.enable_cache(cache_level)
 
         self.protocols = dict()
 
-        log.msg("Starting timekeeping...")
-        self.timestamp = time()
-        self.time = self.world.time
-        self.update_season()
-        self.time_loop = LoopingCall(self.update_time)
-        self.time_loop.start(2)
+    def startFactory(self):
+        log.msg("Initializing factory for world '%s'..." % self.name)
 
         authenticator = configuration.get(self.config_name, "authenticator")
         selected = retrieve_named_plugins(IAuthenticator, [authenticator])[0]
@@ -82,22 +74,57 @@ class BravoFactory(Factory):
         self.handshake_hook = selected.handshake
         self.login_hook = selected.login
 
-        generators = configuration.getlist(self.config_name, "generators")
-        generators = retrieve_sorted_plugins(ITerrainGenerator, generators)
+        # Get our plugins set up.
+        self.register_plugins()
 
-        log.msg("Using generators %s" % ", ".join(i.name for i in generators))
-        self.world.pipeline = generators
+        log.msg("Starting world...")
+        self.world.start()
 
-        automatons = configuration.getlist(self.config_name, "automatons")
-        automatons = retrieve_named_plugins(IAutomaton, automatons)
+        if configuration.has_option(self.config_name, "perm_cache"):
+            cache_level = configuration.getint(self.config_name, "perm_cache")
+            self.world.enable_cache(cache_level)
 
-        log.msg("Using automatons %s" % ", ".join(i.name for i in automatons))
-        self.automatons = automatons
+        log.msg("Starting timekeeping...")
+        self.timestamp = time()
+        self.time = self.world.time
+        self.update_season()
+        self.time_loop = LoopingCall(self.update_time)
+        self.time_loop.start(2)
+
+        # Start automatons.
+        for automaton in self.automatons:
+            automaton.start()
 
         self.chat_consumers = set()
 
         log.msg("Factory successfully initialized for world '%s'!" % self.name)
 
+    def stopFactory(self):
+        """
+        Called before factory stops listening on ports. Used to perform
+        shutdown tasks.
+        """
+
+        log.msg("Shutting down world...")
+
+        # Stop automatons. Technically, they may not actually halt until their
+        # next iteration, but that is close enough for us, probably.
+        # Automatons are contracted to not access the world after stop() is
+        # called.
+        for automaton in self.automatons:
+            automaton.stop()
+
+        self.time_loop.stop()
+
+        # Write back current world time. This must be done before stopping the
+        # world.
+        self.world.time = self.time
+
+        # And now stop the world.
+        self.world.stop()
+
+        log.msg("World data saved!")
+
     def buildProtocol(self, addr):
         """
         Create a protocol.
@@ -122,8 +149,72 @@ class BravoFactory(Factory):
 
         self.register_entity(p)
 
+        # Copy our hooks to the protocol.
+        p.register_hooks()
+
         return p
 
+    def set_username(self, protocol, username):
+        """
+        Attempt to set a new username for a protocol.
+
+        :returns: whether the username was changed
+        """
+
+        # If the username's already taken, refuse it.
+        if username in self.protocols:
+            return False
+
+        if protocol.username in self.protocols:
+            # This protocol's known under another name, so remove it.
+            del self.protocols[protocol.username]
+
+        # Set the username.
+        self.protocols[username] = protocol
+        protocol.username = username
+
+        return True
+
+    def register_plugins(self):
+        """
+        Setup plugin hooks.
+        """
+
+        log.msg("Registering client plugin hooks...")
+
+        plugin_types = {
+            "automatons": IAutomaton,
+            "generators": ITerrainGenerator,
+            "seasons": ISeason,
+            "pre_build_hooks": IPreBuildHook,
+            "post_build_hooks": IPostBuildHook,
+            "dig_hooks": IDigHook,
+            "sign_hooks": ISignHook,
+            "use_hooks": IUseHook,
+        }
+
+        pp = {"factory": self}
+
+        for t, interface in plugin_types.iteritems():
+            l = configuration.getlistdefault(self.config_name, t, [])
+            if issubclass(interface, ISortedPlugin):
+                plugins = retrieve_sorted_plugins(interface, l, parameters=pp)
+            else:
+                plugins = retrieve_named_plugins(interface, l, parameters=pp)
+            log.msg("Using %s: %s" % (t.replace("_", " "),
+                ", ".join(plugin.name for plugin in plugins)))
+            setattr(self, t, plugins)
+
+        # Assign generators to the world pipeline.
+        self.world.pipeline = self.generators
+
+        # Use hooks have special funkiness.
+        uh = self.use_hooks
+        self.use_hooks = defaultdict(list)
+        for plugin in uh:
+            for target in plugin.targets:
+                self.use_hooks[target].append(plugin)
+
     def create_entity(self, x, y, z, name, **kwargs):
         """
         Spawn an entirely new entity.
@@ -211,11 +302,7 @@ class BravoFactory(Factory):
         Update the world's season.
         """
 
-        # Get a sorted list of all the seasons
-        plugins = configuration.getlistdefault(self.config_name,
-            "seasons", [])
-        all_seasons = list(retrieve_named_plugins(ISeason, plugins))
-        all_seasons.sort(key=lambda s: s.day)
+        all_seasons = sorted(self.seasons, key=lambda s: s.day)
 
         # Get all the seasons that we have past the start date of this year.
         # We are looking for the season which is closest to our current day,
@@ -225,10 +312,13 @@ class BravoFactory(Factory):
         if past_seasons:
             # The most recent one is the one we are in
             self.world.season = past_seasons[-1]
-        else:
+        elif all_seasons:
             # We haven't past any seasons yet this year, so grab the last one
             # from 'last year'
             self.world.season = all_seasons[-1]
+        else:
+            # No seasons enabled.
+            self.world.season = None
 
     def chat(self, message):
         """
@@ -242,10 +332,12 @@ class BravoFactory(Factory):
             consumer.write((self, message))
 
         # Prepare the message for chat packeting.
-        for user in self.protocols:
-            message = message.replace(user, chat_name(user))
+        #for user in self.protocols:
+        #    message = message.replace(user, chat_name(user))
         message = sanitize_chat(message)
 
+        log.msg("Chat: %s" % message.encode("utf8"))
+
         packet = make_packet("chat", message=message)
         self.broadcast(packet)
 
@@ -280,6 +372,14 @@ class BravoFactory(Factory):
             if (x, z) in player.chunks:
                 player.transport.write(packet)
 
+    def scan_chunk(self, chunk):
+        """
+        Tell automatons about this chunk.
+        """
+
+        for automaton in self.automatons:
+            automaton.scan(chunk)
+
     def flush_chunk(self, chunk):
         """
         Flush a damaged chunk to all players that have it loaded.
@@ -344,27 +444,6 @@ class BravoFactory(Factory):
             p.player != player):
             yield i.player
 
-    def stopFactory(self):
-        """
-        Called before factory stops listening on ports. Used to perform
-        shutdown tasks.
-        """
-
-        if not self.world.saving:
-            return
-
-        log.msg("Shutting down; flushing world data...")
-
-        # Flush all dirty chunks to disk.
-        for chunk in self.world.dirty_chunk_cache.itervalues():
-            self.world.save_chunk(chunk)
-
-        # Write back current world time.
-        self.world.time = self.time
-        self.world.serializer.save_level(self.world)
-
-        log.msg("World data saved!")
-
     def pauseProducing(self):
         pass
 
