@@ -27,7 +27,7 @@ def pad_to_stride(recipe, rstride, cstride):
 
     pad = (None,) * (cstride - rstride)
     g = grouper(rstride, recipe)
-    padded = list(g.next())
+    padded = list(next(g))
     for row in g:
         padded.extend(pad)
         padded.extend(row)
@@ -39,6 +39,8 @@ class Slot(namedtuple("Slot", "primary, secondary, quantity")):
     A slot in an inventory.
     """
 
+    __slots__ = tuple()
+
     def holds(self, other):
         """
         Whether these slots hold the same item.
@@ -108,6 +110,10 @@ class Inventory(object):
     storage = 0
     holdables = 0
 
+    identifier = 0
+
+    slot_table = tuple()
+
     def __init__(self):
         if self.crafting:
             self.crafting = [None] * self.crafting
@@ -135,12 +141,33 @@ class Inventory(object):
         self.recipe = None
         self.recipe_offset = None
 
-    def __len__(self):
+        # Prep decoder rings.
+        self.slot_encoder_ring = dict(self.slot_table)
+        self.slot_decoder_ring = dict((v, k) for k, v in self.slot_table)
 
+    def __len__(self):
         retval = len(self.crafted) + len(self.crafting) + len(self.armor)
         retval += len(self.storage) + len(self.holdables)
         return retval
 
+    def encode_slot(self, slot):
+        """
+        Turn a server-side slot into a Notchian slot.
+
+        Only used for old serializers.
+        """
+
+        return self.slot_encoder_ring.get(slot, slot)
+
+    def decode_slot(self, slot):
+        """
+        Turn a Notchian slot into a server-side slot.
+
+        Only used for old serializers.
+        """
+
+        return self.slot_decoder_ring.get(slot, slot)
+
     def container_for_slot(self, slot):
         """
         Retrieve the table and index for a given slot.
@@ -173,7 +200,12 @@ class Inventory(object):
         Load data from a packet container.
         """
 
-        items = [None] * len(self.i)
+        length = self.crafting + self.armor + self.storage + self.holdables
+        if self.crafting:
+            # +1 for crafting output slot
+            length += 1
+
+        items = [None] * length
 
         for i, item in enumerate(container.items):
             if item.id < 0:
@@ -252,7 +284,7 @@ class Inventory(object):
         return False
 
 
-    def select_armor(self, index, alternate):
+    def select_armor(self, index, alternate, shift):
         """
         Handle a slot selection on an armor slot.
         """
@@ -296,7 +328,7 @@ class Inventory(object):
         # Yeah, okay, success.
         return True
 
-    def select_crafted(self, index, alternate):
+    def select_crafted(self, index, alternate, shift):
         """
         Handle a slot selection on a crafted output.
         """
@@ -329,7 +361,7 @@ class Inventory(object):
             return False
 
 
-    def select(self, slot, alternate=False):
+    def select(self, slot, alternate=False, shift=False):
         """
         Handle a slot selection.
 
@@ -340,6 +372,7 @@ class Inventory(object):
         :param int slot: which slot was selected
         :param bool alternate: whether the selection is alternate; e.g., if it
                                was done with a right-click
+        :param bool shift: whether the shift key is toogled
         """
 
         # Look up the container and offset.
@@ -352,28 +385,29 @@ class Inventory(object):
             return False
 
         if l is self.armor:
-            return self.select_armor(index, alternate)
+            return self.select_armor(index, alternate, shift)
         elif l is self.crafted:
-            return self.select_crafted(index, alternate)
+            return self.select_crafted(index, alternate, shift)
         elif self.selected is not None and l[index] is not None:
             sslot = self.selected
             islot = l[index]
             if islot.holds(sslot):
                 # both contain the same item
-                if sslot.quantity + islot.quantity <= 64:
-                    # Sum of items fits in one slot, so this is easy.
-                    if alternate:
+                if alternate:
+                    if islot.quantity < 64:
                         l[index] = islot.increment()
                         self.selected = sslot.decrement()
-                    else:
+                else:
+                    if sslot.quantity + islot.quantity <= 64:
+                        # Sum of items fits in one slot, so this is easy.
                         l[index] = islot.increment(sslot.quantity)
                         self.selected = None
-                else:
-                    # fill up slot to 64, move left overs to selection
-                    # valid for left and right mouse click
-                    l[index] = islot.replace(quantity=64)
-                    self.selected = sslot.replace(
-                        quantity=sslot.quantity + islot.quantity - 64)
+                    else:
+                        # fill up slot to 64, move left overs to selection
+                        # valid for left and right mouse click
+                        l[index] = islot.replace(quantity=64)
+                        self.selected = sslot.replace(
+                            quantity=sslot.quantity + islot.quantity - 64)
             else:
                 # Default case: just swap
                 # valid for left and right mouse click
@@ -488,6 +522,11 @@ class Equipment(Inventory):
     storage = 27
     holdables = 9
 
+    slot_table = (
+        (36, 0), (37, 1), (38, 2), (39, 3), (40, 4), (41, 5), (42, 6),
+        (43, 7), (44, 8), (1, 80), (2, 81), (3, 82), (4, 83), (8, 100),
+        (7, 101), (6, 102), (5, 103))
+
     identifier = 0
 
 class Workbench(Inventory):
