@@ -1,4 +1,4 @@
-from collections import namedtuple, defaultdict
+from collections import namedtuple
 from itertools import product, chain
 from time import time
 from urlparse import urlunparse
@@ -16,15 +16,18 @@ from twisted.web.client import getPage
 from bravo.blocks import blocks, items
 from bravo.config import configuration
 from bravo.entity import Sign
+from bravo.errors import BuildError
 from bravo.factories.infini import InfiniClientFactory
-from bravo.ibravo import IChatCommand, IBuildHook, IDigHook, ISignHook, IUseHook
+from bravo.ibravo import (IChatCommand, IPreBuildHook, IPostBuildHook,
+    IDigHook, ISignHook, IUseHook)
 from bravo.inventory import Workbench, sync_inventories
 from bravo.location import Location
 from bravo.motd import get_motd
 from bravo.packets.beta import parse_packets, make_packet, make_error_packet
-from bravo.plugin import retrieve_plugins, retrieve_sorted_plugins, retrieve_named_plugins
+from bravo.plugin import retrieve_plugins
 from bravo.policy.dig import dig_policies
 from bravo.utilities.coords import split_coords
+from bravo.utilities.chat import username_alternatives
 
 (STATE_UNAUTHENTICATED, STATE_CHALLENGED, STATE_AUTHENTICATED) = range(3)
 
@@ -429,28 +432,27 @@ class BravoProtocol(BetaServerProtocol):
         self.config_name = "world %s" % name
 
         log.msg("Registering client hooks...")
-        names = configuration.getlistdefault(self.config_name, "build_hooks",
-            [])
-        self.build_hooks = retrieve_sorted_plugins(IBuildHook, names)
-        names = configuration.getlistdefault(self.config_name, "dig_hooks",
-            [])
-        self.dig_hooks = retrieve_sorted_plugins(IDigHook, names)
-        names = configuration.getlistdefault(self.config_name, "sign_hooks",
-            [])
-        self.sign_hooks = retrieve_sorted_plugins(ISignHook, names)
 
-        names = configuration.getlistdefault(self.config_name, "use_hooks",
-            [])
-        self.use_hooks = defaultdict(list)
-        for plugin in retrieve_named_plugins(IUseHook, names):
-            for target in plugin.targets:
-                self.use_hooks[target].append(plugin)
+        # Retrieve the MOTD. Only needs to be done once.
+        self.motd = configuration.getdefault(self.config_name, "motd", None)
+
+    def register_hooks(self):
+
+        plugin_types = {
+            "pre_build_hooks": IPreBuildHook,
+            "post_build_hooks": IPostBuildHook,
+            "dig_hooks": IDigHook,
+            "sign_hooks": ISignHook,
+            "use_hooks": IUseHook,
+        }
+
+        for t in plugin_types:
+            setattr(self, t, getattr(self.factory, t))
 
         log.msg("Registering policies...")
         self.dig_policy = dig_policies["notchy"]
 
-        # Retrieve the MOTD. Only needs to be done once.
-        self.motd = configuration.getdefault(self.config_name, "motd", None)
+        log.msg("Registered client plugin hooks!")
 
     @inlineCallbacks
     def authenticated(self):
@@ -569,6 +571,21 @@ class BravoProtocol(BetaServerProtocol):
         them depending on the results of the authentication.
         """
 
+        # Check the username. If it's "Player", then the client is almost
+        # certainly cracked, so we'll need to give them a better username.
+        # Thankfully, there's a utility function for finding better usernames.
+        username = container.username
+
+        if username in self.factory.protocols:
+            for name in username_alternatives(username):
+                if name not in self.factory.protocols:
+                    container.username = name
+                    break
+            else:
+                self.error("Your username is already taken.")
+                return
+
+
         if container.protocol < SUPPORTED_PROTOCOL:
             # Kick old clients.
             self.error("This server doesn't support your ancient client.")
@@ -591,8 +608,9 @@ class BravoProtocol(BetaServerProtocol):
 
     def chat(self, container):
         if container.message.startswith("/"):
+            pp = {"factory": self.factory}
 
-            commands = retrieve_plugins(IChatCommand)
+            commands = retrieve_plugins(IChatCommand, parameters=pp)
             # Register aliases.
             for plugin in commands.values():
                 for alias in plugin.aliases:
@@ -613,7 +631,7 @@ class BravoProtocol(BetaServerProtocol):
                                     error.getErrorMessage())
                     )
                 d = maybeDeferred(commands[command].chat_command,
-                                  self.factory, self.username, params)
+                                  self.username, params)
                 d.addCallback(cb)
                 d.addErrback(eb)
             else:
@@ -687,13 +705,13 @@ class BravoProtocol(BetaServerProtocol):
         block = chunk.get_block((smallx, container.y, smallz))
 
         if container.state == "started":
+            tool = self.player.inventory.holdables[self.player.equipped]
             # Check to see whether we should break this block.
-            if self.dig_policy.is_1ko(block):
+            if self.dig_policy.is_1ko(block, tool):
                 self.run_dig_hooks(chunk, coords, blocks[block])
             else:
                 # Set up a timer for breaking the block later.
-                dtime = time() + self.dig_policy.dig_time(block,
-                    self.player.inventory.holdables[self.player.equipped])
+                dtime = time() + self.dig_policy.dig_time(block, tool)
                 self.last_dig = coords, block, dtime
         elif container.state == "stopped":
             # The client thinks it has broken a block. We shall see.
@@ -725,8 +743,7 @@ class BravoProtocol(BetaServerProtocol):
 
         l = []
         for hook in self.dig_hooks:
-            l.append(maybeDeferred(hook.dig_hook,
-                                   self.factory, chunk, x, y, z, block))
+            l.append(maybeDeferred(hook.dig_hook, chunk, x, y, z, block))
 
         dl = DeferredList(l)
         dl.addCallback(lambda none: self.factory.flush_chunk(chunk))
@@ -778,22 +795,37 @@ class BravoProtocol(BetaServerProtocol):
         if container.y == 127 and container.face == '+y':
             return
 
+        # Run pre-build hooks. These hooks are able to interrupt the build
+        # process.
         builddata = BuildData(block, 0x0, container.x, container.y,
             container.z, container.face)
 
-        for hook in self.build_hooks:
-            cont, builddata = yield maybeDeferred(hook.build_hook,
-                self.factory, self.player, builddata)
+        for hook in self.pre_build_hooks:
+            cont, builddata = yield maybeDeferred(hook.pre_build_hook,
+                self.player, builddata)
             if not cont:
                 break
 
+        # Run the build.
+        try:
+            yield maybeDeferred(self.run_build, builddata)
+        except BuildError:
+            return
+
         newblock = builddata.block.slot
+        coords = builddata.x, builddata.y, builddata.z
+
+        # Run post-build hooks. These are merely callbacks which cannot
+        # interfere with the build process, largely because the build process
+        # already happened.
+        for hook in self.post_build_hooks:
+            yield maybeDeferred(hook.post_build_hook, self.player, coords,
+                builddata.block)
 
         # Feed automatons.
         for automaton in self.factory.automatons:
             if newblock in automaton.blocks:
-                automaton.feed(self.factory,
-                    (builddata.x, builddata.y, builddata.z))
+                automaton.feed((builddata.x, builddata.y, builddata.z))
 
         # Re-send inventory.
         # XXX this could be optimized if/when inventories track damage.
@@ -804,6 +836,51 @@ class BravoProtocol(BetaServerProtocol):
         for chunk in self.chunks.itervalues():
             self.factory.flush_chunk(chunk)
 
+    def run_build(self, builddata):
+        block, metadata, x, y, z, face = builddata
+
+        # Don't place items as blocks.
+        if block.slot not in blocks:
+            raise BuildError("Couldn't build item %r as block" % block)
+
+        # Check for orientable blocks.
+        if not metadata and block.orientable():
+            metadata = block.orientation(face)
+            if metadata is None:
+                # Oh, I guess we can't even place the block on this face.
+                raise BuildError("Couldn't orient block %r on face %s" %
+                    (block, face))
+
+        # Make sure we can remove it from the inventory first.
+        if not self.player.inventory.consume((block.slot, 0),
+            self.player.equipped):
+            # Okay, first one was a bust; maybe we can consume the related
+            # block for dropping instead?
+            if not self.player.inventory.consume((block.drop, 0),
+                self.player.equipped):
+                raise BuildError("Couldn't consume %r from inventory" % block)
+
+        # Offset coords according to face.
+        if face == "-x":
+            x -= 1
+        elif face == "+x":
+            x += 1
+        elif face == "-y":
+            y -= 1
+        elif face == "+y":
+            y += 1
+        elif face == "-z":
+            z -= 1
+        elif face == "+z":
+            z += 1
+
+        # Set the block and data.
+        dl = [self.factory.world.set_block((x, y, z), block.slot)]
+        if metadata:
+            dl.append(self.factory.world.set_metadata((x, y, z), metadata))
+
+        return DeferredList(dl)
+
     def equip(self, container):
         self.player.equipped = container.item
 
