@@ -1,10 +1,9 @@
 from __future__ import division
 
 import math
+from itertools import chain, izip, permutations
 from random import Random
 
-from bravo.compat import chain, permutations
-
 SIZE = 2**10
 
 edges2 = list(
@@ -30,21 +29,24 @@ edges3 = list(
 )
 edges3.sort()
 
-def dot(u, v):
+def dot2(u, v):
     """
-    Dot product of two vectors.
+    Dot product of two 2-dimensional vectors.
     """
+    return u[0] * v[0] + u[1] * v[1]
 
-    return sum(i * j for i, j in zip(u, v))
+def dot3(u, v):
+    """
+    Dot product of two 3-dimensional vectors.
+    """
+    return u[0] * v[0] + u[1] * v[1] + u[2] * v[2]
 
 def reseed(seed):
     """
     Reseed the simplex gradient field.
     """
 
-    global p, current_seed
-
-    if current_seed == seed:
+    if seed in fields:
         return
 
     p = range(SIZE)
@@ -52,8 +54,21 @@ def reseed(seed):
     r.seed(seed)
     r.shuffle(p)
     p *= 2
+    fields[seed] = p
+
+def set_seed(seed):
+    """
+    Set the current seed.
+    """
+
+    global current_seed
+
+    reseed(seed)
+
+    current_seed = seed
+
+fields = dict()
 
-p = []
 current_seed = None
 
 f2 = 0.5 * (math.sqrt(3) - 1)
@@ -77,24 +92,25 @@ def simplex2(x, y):
     :raises Exception: the gradient field is not seeded
     """
 
-    if not p:
+    if current_seed is None:
         raise Exception("The gradient field is unseeded!")
 
+    p = fields[current_seed]
+
     # Set up our scalers and arrays.
     coords = [None] * 3
     gradients = [None] * 3
 
-    # XXX ???
     s = (x + y) * f2
-    i = int(math.floor(x + s))
-    j = int(math.floor(y + s))
+    i = math.floor(x + s)
+    j = math.floor(y + s)
     t = (i + j) * g2
     x -= i - t
     y -= j - t
 
     # Clamp to the size of the simplex array.
-    i %= SIZE
-    j %= SIZE
+    i = int(i) % SIZE
+    j = int(j) % SIZE
 
     # Look up coordinates and gradients for each contributing point in the
     # simplex space.
@@ -111,10 +127,10 @@ def simplex2(x, y):
 
     # Do our summation.
     n = 0
-    for coord, gradient in zip(coords, gradients):
+    for coord, gradient in izip(coords, gradients):
         t = 0.5 - coord[0] * coord[0] - coord[1] * coord[1]
         if t > 0:
-            n += t**4 * dot(edges2[gradient % 12], coord)
+            n += t**4 * dot2(edges2[gradient % 12], coord)
 
     # Where's this scaling factor come from?
     return n * 70
@@ -138,27 +154,28 @@ def simplex3(x, y, z):
                        function somehow
     """
 
-    if not p:
+    if current_seed is None:
         raise Exception("The gradient field is unseeded!")
 
+    p = fields[current_seed]
+
     f = 1 / 3
     g = 1 / 6
     coords = [None] * 4
     gradients = [None] * 4
 
     s = (x + y + z) * f
-    i = int(math.floor(x + s))
-    j = int(math.floor(y + s))
-    k = int(math.floor(z + s))
+    i = math.floor(x + s)
+    j = math.floor(y + s)
+    k = math.floor(z + s)
     t = (i + j + k) * g
-    unskewed = i - t, j - t, k - t
     x -= i - t
     y -= j - t
     z -= k - t
 
-    i %= SIZE
-    j %= SIZE
-    k %= SIZE
+    i = int(i) % SIZE
+    j = int(j) % SIZE
+    k = int(k) % SIZE
 
     # Do the coord and gradient lookups. Unrolled for speed and clarity.
     # These should be + 2 * g, but instead we do + f because we already have
@@ -208,10 +225,11 @@ def simplex3(x, y, z):
     gradients[3] = p[i + 1 + p[j + 1 + p[k + 1]]]
 
     n = 0
-    for coord, gradient in zip(coords, gradients):
-        t = 0.6 - coord[0] * coord[0] - coord[1] * coord[1] - coord[2] * coord[2]
+    for coord, gradient in izip(coords, gradients):
+        t = (0.6 - coord[0] * coord[0] - coord[1] * coord[1] - coord[2] *
+            coord[2])
         if t > 0:
-            n += t**4 * dot(edges2[gradient % 12], coord)
+            n += t**4 * dot3(edges2[gradient % 12], coord)
 
     # Where's this scaling factor come from?
     return n * 32
