@@ -8,9 +8,11 @@ import bravo.factories.beta
 
 class MockProtocol(object):
 
+    username = None
+
     def __init__(self, player):
         self.player = player
-        self.location = player.location
+        self.location = player.location if player else None
 
 class TestBravoFactory(unittest.TestCase):
 
@@ -19,11 +21,7 @@ class TestBravoFactory(unittest.TestCase):
         self.name = "unittest"
 
         bravo.config.configuration.add_section("world unittest")
-        d = {
-            "port"          : "0",
-        }
-        for k, v in d.items():
-            bravo.config.configuration.set("world unittest", k, v)
+        bravo.config.configuration.set("world unittest", "port", "0")
 
         self.f = bravo.factories.beta.BravoFactory(self.name)
 
@@ -45,6 +43,84 @@ class TestBravoFactory(unittest.TestCase):
 
         self.assertEqual(self.f.eid, 1)
 
+    def test_update_season_empty(self):
+        """
+        If no seasons are enabled, things should proceed as normal.
+        """
+
+        bravo.config.configuration.set("world unittest", "seasons", "")
+        self.f.register_plugins()
+
+        self.f.day = 0
+        self.f.update_season()
+        self.assertTrue(self.f.world.season is None)
+
+        self.f.day = 90
+        self.f.update_season()
+        self.assertTrue(self.f.world.season is None)
+
+    def test_update_season_winter(self):
+        """
+        If winter is the only season available, then only winter should be
+        selected, regardless of day.
+        """
+
+        bravo.config.configuration.set("world unittest", "seasons", "winter")
+        self.f.register_plugins()
+
+        self.f.day = 0
+        self.f.update_season()
+        self.assertEqual(self.f.world.season.name, "winter")
+
+        self.f.day = 90
+        self.f.update_season()
+        self.assertEqual(self.f.world.season.name, "winter")
+
+    def test_update_season_switch(self):
+        """
+        The season should change from spring to winter when both are enabled.
+        """
+
+        bravo.config.configuration.set("world unittest", "seasons",
+            "winter, spring")
+        self.f.register_plugins()
+
+        self.f.day = 0
+        self.f.update_season()
+        self.assertEqual(self.f.world.season.name, "winter")
+
+        self.f.day = 90
+        self.f.update_season()
+        self.assertEqual(self.f.world.season.name, "spring")
+
+    def test_set_username(self):
+        p = MockProtocol(None)
+        p.username = "Hurp"
+        self.f.protocols["Hurp"] = p
+
+        self.assertTrue(self.f.set_username(p, "Derp"))
+
+        self.assertTrue("Derp" in self.f.protocols)
+        self.assertTrue("Hurp" not in self.f.protocols)
+        self.assertEqual(p.username, "Derp")
+
+    def test_set_username_taken(self):
+        p = MockProtocol(None)
+        p.username = "Hurp"
+        self.f.protocols["Hurp"] = p
+        self.f.protocols["Derp"] = None
+
+        self.assertFalse(self.f.set_username(p, "Derp"))
+
+        self.assertEqual(p.username, "Hurp")
+
+    def test_set_username_noop(self):
+        p = MockProtocol(None)
+        p.username = "Hurp"
+        self.f.protocols["Hurp"] = p
+
+        self.assertFalse(self.f.set_username(p, "Hurp"))
+
 class TestBravoFactoryStarted(unittest.TestCase):
     """
     Tests which require ``startFactory()`` to be called.
@@ -75,8 +151,7 @@ class TestBravoFactoryStarted(unittest.TestCase):
     def tearDown(self):
         bravo.config.configuration.remove_section("world unittest")
 
-        self.f.world.chunk_management_loop.stop()
-        self.f.time_loop.stop()
+        self.f.stopFactory()
 
         shutil.rmtree(self.d)
 
@@ -138,21 +213,3 @@ class TestBravoFactoryStarted(unittest.TestCase):
         for player, radius, result in expected_results:
             found = [p.eid for p in self.f.players_near(player, radius)]
             self.assertEqual(set(found), set(result))
-
-    def test_update_season_simple(self):
-        self.f.day = 90
-        self.f.update_season()
-        self.assertEqual(self.f.world.season.name, "spring")
-
-        self.f.day = 0
-        self.f.update_season()
-        self.assertEqual(self.f.world.season.name, "winter")
-
-    def test_update_season_advanced(self):
-        self.f.day = 0
-        self.f.update_season()
-        self.assertEqual(self.f.world.season.name, "winter")
-
-        self.f.day = 92
-        self.f.update_season()
-        self.assertEqual(self.f.world.season.name, "spring")
